#include "mem_ctx.h"
#include "aliases.h"
#include "mem_arena.h"
#include <assert.h>
#include <stdlib.h>

typedef struct mem_ctx MemCTX;
struct mem_ctx {
  Arena *main;
  Arena *temp;
  bool main_initialised;
  bool temp_initialised;
};

internal MemCTX g_context = {0};

internal Arena *get_arena(CTXDestBuffer buffer);

void wapp_mem_ctx_init(u64 main_buf_capacity, u64 temp_buf_capacity) {
  g_context.main_initialised =
      wapp_mem_arena_init(&g_context.main, main_buf_capacity);
  g_context.temp_initialised =
      wapp_mem_arena_init(&g_context.temp, temp_buf_capacity);
}

void *wapp_mem_ctx_alloc(CTXDestBuffer buffer, u64 size) {
  Arena *arena = get_arena(buffer);
  if (!arena) {
    return NULL;
  }

  return wapp_mem_arena_alloc(arena, size);
}

void *wapp_mem_ctx_alloc_aligned(CTXDestBuffer buffer, u64 size,
                                 u64 alignment) {
  Arena *arena = get_arena(buffer);
  if (!arena) {
    return NULL;
  }

  return wapp_mem_arena_alloc_aligned(arena, size, alignment);
}

void *wapp_mem_ctx_realloc(CTXDestBuffer buffer, void *ptr, u64 size) {
  Arena *arena = get_arena(buffer);
  if (!arena) {
    return NULL;
  }

	return wapp_mem_arena_realloc(arena, ptr, size);
}

void *wapp_mem_ctx_realloc_aligned(CTXDestBuffer buffer, void *ptr, u64 size,
																	 u64 alignment) {
  Arena *arena = get_arena(buffer);
  if (!arena) {
    return NULL;
  }

	return wapp_mem_arena_realloc_aligned(arena, ptr, size, alignment);
}

void wapp_mem_ctx_clear(CTXDestBuffer buffer) {
  Arena *arena = get_arena(buffer);
  if (!arena) {
    return;
  }

  wapp_mem_arena_clear(arena);
}

void wapp_mem_ctx_free(void) {
  wapp_mem_arena_free(&(g_context.main));
  g_context.main_initialised = false;

  wapp_mem_arena_free(&(g_context.temp));
  g_context.temp_initialised = false;
}

internal Arena *get_arena(CTXDestBuffer buffer) {
  Arena *output = NULL;

  switch (buffer) {
  case CTX_DEST_BUFFER_MAIN:
    if (g_context.main_initialised) {
      output = g_context.main;
    }
    break;
  case CTX_DEST_BUFFER_TEMP:
    if (g_context.temp_initialised) {
      output = g_context.temp;
    }
    break;
  default:
    assert(false && "Not all context destination buffers are handled");
    break;
  }

  return output;
}
